@@ -62,6 +62,9 @@ struct uwsgi_option http_options[] = {
 	{"http-server-name-as-http-host", required_argument, 0, "force SERVER_NAME to HTTP_HOST", uwsgi_opt_true, &uhttp.server_name_as_http_host, 0},
 	{"http-headers-timeout", required_argument, 0, "set internal http socket timeout for headers", uwsgi_opt_set_int, &uhttp.headers_timeout, 0},
 	{"http-connect-timeout", required_argument, 0, "set internal http socket timeout for backend connections", uwsgi_opt_set_int, &uhttp.connect_timeout, 0},
+
+	{"http-manage-source", no_argument, 0, "manage the SOURCE HTTP method placing the session in raw mode", uwsgi_opt_true, &uhttp.manage_source, 0},
+	{"http-enable-proxy-protocol", optional_argument, 0, "manage PROXY protocol requests", uwsgi_opt_true, &uhttp.enable_proxy_protocol, 0},
 	{0, 0, 0, 0, 0, 0, 0},
 };
 
@@ -150,10 +153,14 @@ static int http_add_uwsgi_header(struct corerouter_peer *peer, char *hh, size_t
 	}
 
 	else if (!uwsgi_strncmp("CONNECTION", 10, hh, keylen)) {
-		if (!uwsgi_strnicmp(val, vallen, "close", 5)) {
+		if (!uwsgi_strnicmp(val, vallen, "close", 5) || !uwsgi_strnicmp(val, vallen, "upgrade", 7)) {
 			hr->session.can_keepalive = 0;
 		}
 	}
+	else if (peer->key == uwsgi.hostname && hr->raw_body && !uwsgi_strncmp("ICE_URL", 7, hh, keylen)) {
+		peer->key = val;
+		peer->key_len = vallen;
+	}
 
 #ifdef UWSGI_ZLIB
 	else if (uhttp.auto_gzip && !uwsgi_strncmp("ACCEPT_ENCODING", 15, hh, keylen)) {
@@ -189,6 +196,14 @@ int http_headers_parse(struct corerouter_peer *peer) {
 	char *watermark = ptr + hr->headers_size;
 	char *base = ptr;
 	char *query_string = NULL;
+	char *proxy_src = NULL;
+	char *proxy_dst = NULL;
+	char *proxy_src_port = NULL;
+	char *proxy_dst_port = NULL;
+	uint16_t proxy_src_len = 0;
+	uint16_t proxy_dst_len = 0;
+	uint16_t proxy_src_port_len = 0;
+	uint16_t proxy_dst_port_len = 0;
 
 	peer->out = uwsgi_buffer_new(uwsgi.page_size);
 	// force this buffer to be destroyed as soon as possibile
@@ -201,10 +216,19 @@ int http_headers_parse(struct corerouter_peer *peer) {
 	struct uwsgi_buffer *out = peer->out;
 	int found = 0;
 
+        if (uwsgi.enable_proxy_protocol || uhttp.enable_proxy_protocol) {
+		ptr = proxy1_parse(ptr, watermark, &proxy_src, &proxy_src_len, &proxy_dst, &proxy_dst_len, &proxy_src_port, &proxy_src_port_len, &proxy_dst_port, &proxy_dst_port_len);
+		base = ptr;
+        }
+
 	// REQUEST_METHOD 
 	while (ptr < watermark) {
 		if (*ptr == ' ') {
 			if (uwsgi_buffer_append_keyval(out, "REQUEST_METHOD", 14, base, ptr - base)) return -1;
+			// on SOURCE METHOD, force raw body
+			if (uhttp.manage_source && !uwsgi_strncmp(base, ptr - base, "SOURCE", 6)) {
+				hr->raw_body = 1;
+			}
 			ptr++;
 			found = 1;
 			break;
@@ -324,12 +348,21 @@ int http_headers_parse(struct corerouter_peer *peer) {
 	}
 
 #ifdef UWSGI_SSL
-	if (hr_https_add_vars(hr, out)) return -1;
+	if (hr_https_add_vars(hr, peer, out)) return -1;
 #endif
 
 	// REMOTE_ADDR
-	if (uwsgi_buffer_append_keyval(out, "REMOTE_ADDR", 11, peer->session->client_address, strlen(peer->session->client_address))) return -1;
-	if (uwsgi_buffer_append_keyval(out, "REMOTE_PORT", 11, peer->session->client_port, strlen(peer->session->client_port))) return -1;
+        if (proxy_src) {
+		if (uwsgi_buffer_append_keyval(out, "REMOTE_ADDR", 11, proxy_src, proxy_src_len)) return -1;
+		if (proxy_src_port) {
+			if (uwsgi_buffer_append_keyval(out, "REMOTE_PORT", 11, proxy_src_port, proxy_src_port_len)) return -1;
+		}
+	}
+	else
+	{
+		if (uwsgi_buffer_append_keyval(out, "REMOTE_ADDR", 11, peer->session->client_address, strlen(peer->session->client_address))) return -1;
+		if (uwsgi_buffer_append_keyval(out, "REMOTE_PORT", 11, peer->session->client_port, strlen(peer->session->client_port))) return -1;
+	}
 
 	//HEADERS
 	base = ptr;
@@ -796,6 +829,13 @@ ssize_t http_parse(struct corerouter_peer *main_peer) {
 				if (uwsgi_buffer_append(new_peer->out, main_peer->in->buf + hr->headers_size + 1, hr->remains)) return -1;
 			}
 
+			if (hr->websockets > 2 && hr->websocket_key_len > 0) {
+				hr->raw_body = 1;
+			}
+
+			// on raw body, ensure keepalive is disabled
+			if (hr->raw_body) hr->session.can_keepalive = 0;
+
 			if (hr->session.can_keepalive && hr->content_length == 0) {
 				main_peer->disabled = 1;
 				// stop reading from the client
@@ -807,9 +847,7 @@ ssize_t http_parse(struct corerouter_peer *main_peer) {
 				break;
         		}
 
-			if (hr->websockets > 2 && hr->websocket_key_len > 0) {
-				hr->raw_body = 1;
-			}
+
 			new_peer->can_retry = 1;
 			// reset main timeout
 			http_set_timeout(main_peer, uhttp.cr.socket_timeout);
